@@ -45,7 +45,7 @@ use constant {
     IPV6MIB => 3,
 };
 
-$VERSION = '3.92';
+$VERSION = '3.974000';
 
 
 
@@ -94,6 +94,75 @@ $VERSION = '3.92';
     'i6_n2p_phys_addr'  => \&SNMP::Info::munge_mac,
 );
 
+sub parse_inetaddress {
+    my ($info, $inetaddress) = @_;
+    my @parts = split(/\./, $inetaddress);
+    # first part is the InetAddressType
+    my $addrtype = shift @parts;
+    # second part is the number of octets
+    my $addrsize = shift @parts;
+    # InetAddressIPv4
+    if ($addrtype == 1) {
+        if ($addrsize == 4) {
+            my @ipv4 = @parts[0..3];
+            return {
+                type    => $addrtype,
+                address => join('.', @ipv4),
+            };
+        }
+        elsif ($info->debug()) {
+            print("invalid number of octets '$addrsize' for InetAddressIPv4 '$inetaddress'\n");
+        }
+    }
+    # InetAddressIPv6
+    if ($addrtype == 2) {
+        if ($addrsize == 16) {
+            my @ipv6 = @parts[0..15];
+            return {
+                type    => $addrtype,
+                address => join(':', unpack('(H4)*', pack('C*', @ipv6))),
+            };
+        }
+        elsif ($info->debug()) {
+            print("invalid number of octets '$addrsize' for InetAddressIPv4 '$inetaddress'\n");
+        }
+    }
+    # InetAddressIPv4z
+    elsif ($addrtype == 3) {
+        if ($addrsize == 8) {
+            my @ipv4 = @parts[0..3];
+            my @zone_identifier = @parts[4..7];
+            return {
+                type            => $addrtype,
+                address         => join('.', @ipv4),
+                zone_identifier => unpack("L", pack("C*", @zone_identifier)),
+            };
+        }
+        elsif ($info->debug()) {
+            print("invalid number of octets '$addrsize' for InetAddressIPv4z '$inetaddress'\n");
+        }
+    }
+    # InetAddressIPv6z
+    elsif ($addrtype == 4) {
+        if ($addrsize == 20) {
+            my @ipv6 = @parts[0..15];
+            my @zone_identifier = @parts[16..19];
+            return {
+                type            => $addrtype,
+                address         => join(':', unpack('(H4)*', pack('C*', @ipv6))),
+                zone_identifier => unpack("L", pack("C*", @zone_identifier)),
+            };
+        }
+        elsif ($info->debug()) {
+            print("invalid number of octets '$addrsize' for InetAddressIPv6z '$inetaddress'\n");
+        }
+    }
+    elsif ($info->debug()) {
+        print("unsupported address type '$addrtype'\n");
+    }
+
+    return;
+}
 
 sub ipv6_n2p_mac {
     my $info = shift;
@@ -251,17 +320,19 @@ sub ipv6_index {
     my $info = shift;
     my $return;
     my $ipv6_index = &_test_methods( $info, {
-	ip_addr6_index  => IPMIB,
-	c_addr6_index   => CISCO,
-				    });
+        ip_addr6_index  => IPMIB,
+        c_addr6_index   => CISCO,
+    });
     return unless defined $ipv6_index;
-    foreach my $row (keys %$ipv6_index){
-        if ($row =~ /^(\d+)\.([\d\.]+)$/) {
-            my $addrtype = $1; my $v6addr = $2;
-            if ($addrtype == 2) { # IPv6
-		$return->{$row} = $ipv6_index->{$row};
-	    }
-	}
+    foreach my $row (keys %$ipv6_index) {
+        my $parsed_address = $info->parse_inetaddress($row);
+        next
+            unless defined $parsed_address;
+        # only IPv6 and IPv6z (with zone identifier)
+        next
+            unless $parsed_address->{type} == 2 || $parsed_address->{type} == 4;
+
+        $return->{$row} = $ipv6_index->{$row};
     }
     printf("%s: data comes from %s.\n", &_my_sub_name, $info->_method_used() ) if $info->debug();
     return $return;
@@ -271,17 +342,19 @@ sub ipv6_type {
     my $info = shift;
     my $return;
     my $ipv6_type = &_test_methods( $info, {
-	ip_addr6_type  => IPMIB,
-	c_addr6_type   => CISCO,
-				    });
+        ip_addr6_type  => IPMIB,
+        c_addr6_type   => CISCO,
+    });
     return unless defined $ipv6_type;
     foreach my $row (keys %$ipv6_type){
-        if ($row =~ /^(\d+)\.([\d\.]+)$/) {
-            my $addrtype = $1; my $v6addr = $2;
-            if ($addrtype == 2) { # IPv6
-		$return->{$row} = $ipv6_type->{$row};
-	    }
-	}
+        my $parsed_address = $info->parse_inetaddress($row);
+        next
+            unless defined $parsed_address;
+        # only IPv6 and IPv6z (with zone identifier)
+        next
+            unless $parsed_address->{type} == 2 || $parsed_address->{type} == 4;
+
+         $return->{$row} = $ipv6_type->{$row};
     }
     printf("%s: data comes from %s.\n", &_my_sub_name, $info->_method_used() ) if $info->debug();
     return $return;
@@ -291,17 +364,18 @@ sub ipv6_pfx_origin {
     my $info = shift;
     my $return;
     my $ipv6_pfx_origin = &_test_methods( $info, {
-	ip_pfx_origin  => IPMIB,
-	c_pfx_origin   => CISCO,
-				    });
+        ip_pfx_origin  => IPMIB,
+        c_pfx_origin   => CISCO,
+    });
     return unless defined $ipv6_pfx_origin;
-    foreach my $row (keys %$ipv6_pfx_origin){
+    foreach my $row (keys %$ipv6_pfx_origin) {
         if ($row =~ /^(\d+)\.(\d+)\.([\d\.]+)\.(\d+)$/) {
             my $ifindex = $1; my $type = $2; my $pfx = $3; my $len = $4;
-            if ($type == 2) { # IPv6
-		$return->{$row} = $ipv6_pfx_origin->{$row};
-	    }
-	}
+            # only IPv6 and IPv6z (with zone identifier)
+            if ($type == 2 || $type == 4) {
+                $return->{$row} = $ipv6_pfx_origin->{$row};
+            }
+        }
     }
     printf("%s: data comes from %s.\n", &_my_sub_name, $info->_method_used() ) if $info->debug();
     return $return;
@@ -311,24 +385,26 @@ sub ipv6_addr_prefix {
     my $info = shift;
     my $return;
     my $ipv6_addr_prefix = &_test_methods( $info, {
-	ip_addr6_pfx  => IPMIB,
-	c_addr6_pfx   => CISCO,
-				    });
+        ip_addr6_pfx  => IPMIB,
+        c_addr6_pfx   => CISCO,
+    });
     return unless defined $ipv6_addr_prefix;
-    foreach my $row (keys %$ipv6_addr_prefix){
-        if ($row =~ /^(\d+)\.[\d\.]+$/) {
-            my $type = $1;
-	    if (($type == 2) or ($type == 4)) { # IPv6
-		# Remove interface specific part from vrf interfaces
-		if ($row =~ /^((\d+\.){17}\d+)/) { $row = $1 }
-		# Remove the OID part from the value
-		my $val = $ipv6_addr_prefix->{$row};
-		if ( $val =~ /^.+?((?:\d+\.){19}\d+)$/ ){
-		    $val = $1;
-		    $return->{$row} = $val;
-		}
-	    }
-	}
+    foreach my $row (keys %$ipv6_addr_prefix) {
+        my $parsed_address = $info->parse_inetaddress($row);
+        next
+            unless defined $parsed_address;
+        # only IPv6 and IPv6z (with zone identifier)
+        next
+            unless $parsed_address->{type} == 2 || $parsed_address->{type} == 4;
+
+        # Remove interface specific part from vrf interfaces
+        if ($parsed_address->{type} == 4 && $row =~ /^((\d+\.){17}\d+)/) { $row = $1 }
+        # Remove the OID part from the value
+        my $val = $ipv6_addr_prefix->{$row};
+        if ( $val =~ /^.+?((?:\d+\.){19}\d+)$/ ){
+            $val = $1;
+            $return->{$row} = $val;
+        }
     }
     printf("%s: data comes from %s.\n", &_my_sub_name, $info->_method_used() ) if $info->debug();
     return $return;
@@ -345,7 +421,7 @@ sub ipv6_addr_prefixlength {
     foreach my $row (keys %$ipv6_addr_prefix) {
         if ($row =~ /^(\d+)\.[\d\.]+$/) {
             my $type = $1;
-            if (($type == 2) or ($type == 4)) { # IPv6
+            if ($type == 2 || $type == 4) { # IPv6 or IPv6z (with zone identifier)
                 # Remove interface specific part from vrf interfaces
                 if ($row =~ /^((\d+\.){17}\d+)/) { $row = $1 }
                 # Remove the OID part from the value
@@ -366,20 +442,14 @@ sub ipv6_addr {
     my $return = {};
     my $indexes = $info->ipv6_index();
     foreach my $row (keys %$indexes) {
-        my @parts = split(/\./, $row);
-        my $is_valid = 0;
-        if (scalar @parts == 18) {
-            my $addrtype = shift @parts;
-            $is_valid = 1;
-        } elsif (scalar @parts == 17) {
-            $is_valid = 1;
-        }
-        my $addrsize = shift @parts; # First element now is addrsize, should be 16
-        if ($is_valid && $addrsize == 16) {
-            $return->{$row} = join(':', unpack('(H4)*', pack('C*', @parts)));
-        } elsif ($info->debug()) {
-            printf("%s: unable to decode table index to IPv6 address. Raw data is [%s].\n", &_my_sub_name, $row);
-        }
+        my $parsed_address = $info->parse_inetaddress($row);
+        next
+            unless defined $parsed_address;
+        # only IPv6 and IPv6z (with zone identifier)
+        next
+            unless $parsed_address->{type} == 2 || $parsed_address->{type} == 4;
+
+        $return->{$row} = $parsed_address->{address};
     }
     return $return;
 }
@@ -458,7 +528,7 @@ Jeroen van Ingen and Carlos Vicente
 
 =head1 DESCRIPTION
 
-The SNMP::Info::IPv6 class implements functions to for mapping IPv6 addresses
+The SNMP::Info::IPv6 class implements functions for mapping IPv6 addresses
 to MAC addresses, interfaces and more. It will use data from the F<IP-MIB>,
 F<IPV6-MIB>, or the F<CISCO-IETF-IP-MIB>, whichever is supported by the
 device.
@@ -469,7 +539,7 @@ device classes.
 For debugging purposes you can call this class directly as you would
 SNMP::Info
 
- my $info = new SNMP::Info::IPv6 (...);
+ my $info = SNMP::Info::IPv6->new(...);
 
 =head2 Inherited Classes
 
@@ -555,6 +625,13 @@ Maps an address of type C<cInetNetToMediaNetAddressType> on interface C<ifIndex>
 Takes an octet stream (HEX-STRING) and returns a colon separated ASCII hex
 string.
 
+=item parse_inetaddress
+
+Parses an InetAddress as defined by INET-ADDRESS-MIB.
+
+Returns a hashref with the type as defined by InetAddressType, the textual
+address and optionally the zone_identifier.
+
 =back
 
 =cut
